// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file stepper.h
// File description here
// 
// @brief Stepper driver
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include <Arduino.h>

// If the MMI was enabled use the FastIO from the library
#ifdef _MMI_ENABLED_
#include <FastIO.h>
#else
#include "FastIO.h"
#endif

#include "stepper.h"


// CONSTANTS AND DEFINITIONS
// ---------------------------------------------------------------------------


// CLASS CONSTRUCTORS
// ---------------------------------------------------------------------------
// Constructor
stepper::stepper ( uint8_t stepPin, uint8_t dirPin, uint8_t enPin, uint8_t pulseW )
{
   _stepReg   = fio_pinToOutputRegister ( stepPin, LOW );
   _stepPin   = fio_pinToBit ( stepPin );

   _dirReg    = fio_pinToOutputRegister ( dirPin, LOW );
   _dirPin    = fio_pinToBit ( dirPin );
   _enPin     = enPin;
   _pulseW    = pulseW;
   _numSteps  = 0;
   
   pinMode ( _stepPin, OUTPUT );
   pinMode ( _dirPin, OUTPUT );
   
   // Only configure the enable control pin if the pin is not 0
   // ---------------------------------------------------------
   if ( _enPin != 0 )
   {
      pinMode ( _enPin, OUTPUT );
   }
   
   fio_digitalWrite ( _stepReg, _stepPin, LOW);
   fio_digitalWrite ( _dirReg, _dirPin, stepper::CW );

   // Only configure the enable control pin if the pin is not 0
   // ---------------------------------------------------------
   if ( _enPin != 0 )
   {
      digitalWrite ( _enPin, DISABLE );      
   }
}

// PUBLIC METHODS
// ---------------------------------------------------------------------------
void stepper::runStep ( )
{
   // Take new step is counter is still not 0
   // ---------------------------------------
   if ( _numSteps > 0 )
   {
      step();
      _numSteps--;
   }
}

void stepper::play ( void )
{
   while ( _numSteps-- > 0 )
   {
      step();
   }
}

void stepper::step ( )
{
   fio_digitalWrite_HIGH ( _stepReg, _stepPin );
   delayMicroseconds ( _pulseW );
   fio_digitalWrite_LOW ( _stepReg, _stepPin );
}


void stepper::setSteps ( uint16_t numSteps )
{
   _numSteps = numSteps;
}

uint16_t stepper::getSteps ( void )
{
   return ( _numSteps );
}

void stepper::setDirection ( t_dir direction )
{
   fio_digitalWrite ( _dirReg, _dirPin, direction );
}

void stepper::enable ( void )
{
   // Enable if configured enable pin different than 0
   // -------------------------------------------------
   if ( _enPin != 0 )
   {
      digitalWrite ( _enPin, ENABLE );      
   }
}

void stepper::disable ( void )
{
   // Disable if configured enable pin different than 0
   // -------------------------------------------------
   if ( _enPin != 0 )
   {
      digitalWrite ( _enPin, DISABLE );      
   }
}

// PRIVATE METHODS
// ---------------------------------------------------------------------------